package junit.extensions.eclipse.diff;

import org.eclipse.jdt.internal.junit.ui.TestRunInfo;
import org.eclipse.jdt.junit.ITestRunListener;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

public class TestRunListener implements ITestRunListener {

    private interface PageUpdater {
        public void update(IWorkbenchPage page) throws Exception;
    }

    private void updatePage(final PageUpdater updater) {
        IWorkbenchWindow window = getWindow();
        if (window == null) {
            JUnitDiffPlugin.getDefault().logSystemErrorMessage("cannot found workbench window.", this);
            return;
        }
        final IWorkbenchPage page = window.getActivePage();
        if (page == null) {
            JUnitDiffPlugin.getDefault().logSystemErrorMessage("cannot found workbench page.", this);
            return;
        }
        window.getShell().getDisplay().asyncExec(new Runnable() {
            public void run() {
                try {
                    updater.update(page);
                } catch (Exception e) {
                    JUnitDiffPlugin.getDefault().logSystemError(e, TestRunListener.this);
                }
            }
        });
    }

    public void testRunStarted(int testCount) {
        updatePage(new PageUpdater() {
            public void update(IWorkbenchPage page) {
                IViewPart diffView = page.findView(JUnitDiffView.ID);
                if (diffView != null) {
                    ((JUnitDiffView) diffView).testStarted();
                }
            }
        });
    }

    public void testRunEnded(long elapsedTime) {
    }

    private IWorkbenchWindow getWindow() {
        IWorkbench workbench = PlatformUI.getWorkbench();
        IWorkbenchWindow window = workbench.getActiveWorkbenchWindow();
        if (window == null) {
            IWorkbenchWindow[] windows = workbench.getWorkbenchWindows();
            if (windows.length > 0)
                window = windows[0];
        }
        return window;
    }

    public void testRunStopped(long elapsedTime) {
    }

    public void testStarted(String testId, String testName) {
    }

    public void testEnded(String testId, String testName) {
    }

    public void testFailed(int status, final String testId, final String testName, final String trace) {

        TestRunInfo info = new TestRunInfo(testId, testName);
        info.setTrace(trace);
        final JUnitDiff jnitDiff = new JUnitDiff(info);

        updatePage(new PageUpdater() {
            public void update(IWorkbenchPage page) throws Exception {
                IViewPart diffView = page.findView(JUnitDiffView.ID);
                if (diffView == null) {
                    IWorkbenchPart activePart = page.getActivePart();
                    diffView = page.showView(JUnitDiffView.ID);
                    //restore focus stolen by the creation of the console
                    page.activate(activePart);
                }
                ((JUnitDiffView) diffView).testChanged(jnitDiff);
            }
        });
    }

    public void testRunTerminated() {
    }

    public void testReran(String testId, String testClass, String testName, int status, String trace) {
    }
}
